// This file defines the dialog which is used when the user creates a
// new command, or edits an existing one. It also defines the "Select
// application" dialog used by that dialog.

#ifndef NEW_COMMAND_DIALOG_H
#define NEW_COMMAND_DIALOG_H

#include <eikcfdlg.h>
#include "VoiceControlDialog.h"

// The dialog class. It's a standard dialog, which also is capable of being
// notified of changes in state of the audio recorder utility object.
class CNewCommandDialog : public CEikDialog, public MMdaObjectStateChangeObserver
	{
public:
	CNewCommandDialog(CVoiceControlDialog& aParent, TInt aResourceFileId,
		TBool aEditingCommand);
	virtual ~CNewCommandDialog();

private:
	// from CEikDialog

	// This function will be called before the dialog is laid out.
	virtual void PreLayoutDynInitL();
	// This will be called when the user wants to close the dialog.
	virtual TBool OkToExitL(TInt aButtonId);
	// This will be called when a key is pressed/released.
	TKeyResponse OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType);
	// This is called when the current dialog line changes
	virtual void LineChangedL(TInt aControlId);
	
	// from MMdaObjectStateChangeObserver
	// This will be called when the audio recorder's state changes.
	void MoscoStateChangeEvent(CBase* aObject, TInt aPreviousState, TInt aCurrentState, TInt aErrorCode);

	// from MCoeControlObserver
	// This will be called when the user does something to a control.
	virtual void HandleControlEventL(CCoeControl* aControl, TCoeEvent aEventType);

	// Records a new command.
	void RecordL();
	// Re-records an existing command.
	void RerecordL();
	// Stops recording and closes the dialog.
	void StopRecordingL();
	// Compresses a voice data file and generates information about it.
	TInt ConvertVoiceFileL();
	// Modifies the contents of an existing voice command.
	void ModifyCommandInListL();
	
	// A reference to the parent "Voice Control" dialog.
	CVoiceControlDialog& iParent;

	// A pointer to the command this dialog is operating on.
	CVoiceCommand* iVoiceCommand;

	// The current audio recorder state.
	TInt iState;
	// The number of repetitions of a command that have been recorded.
	TInt iNumRecorded;
	// The ID of the resource file used to create this dialog.
	TInt iResourceFileId;
	// Whether the dialog is being used to create a new command or edit
	// an existing one.
	TBool iEditingCommand;
	};

// The "Select application" dialog is very simple. It just contains
// a list of all applications in the system.
class CSelectApplicationDialog : public CEikDialog
	{
public:
	// C++ constructor. The caller passes in a reference to a path,
	// which will be updated.
	CSelectApplicationDialog(TDes& aApplicationPath);
	~CSelectApplicationDialog();

private:
	// Called before the dialog is laid out.
	virtual void PreLayoutDynInitL();
	// This will be called when the user wants to close the dialog.
	virtual TBool OkToExitL(TInt aButtonId);

	// An array of the paths of all the applications in the system.
	CDesCArray* iApplicationPaths;
	// The selected app path, updated before the dialog is closed.
	TDes& iSelectedApplicationPath;
	// A pointer to the list box containing app captions.
	CEikTextListBox* iListBox;
	};

#endif