// This file defines the main "Voice Control" dialog, which displays a list
// of commands and allows the user to create new ones. It also includes
// settings for the system.

#ifndef __VOICECONTROL_DIALOG_H
#define __VOICECONTROL_DIALOG_H

#include <coeccntx.h>
#include <eikenv.h>
#include <eikdialg.h>
#include <bautils.h>
#include <eikon.hrh>
#include <eikbtgpc.h>
#include <eikchlst.h>
#include <eiktxlbx.h>
#include <eiktxlbm.h>
#include <s32file.h>
#include <apgtask.h>
#include <apgcli.h>
#include <gulalign.h>
#include <eiklabel.h>

#include <VoiceControl.rsg>
#include "VoiceControl.hrh"

#include "VoiceRec.h"

// Include the context-sensitive help file.
#include "VoiceControl.hlp.hrh"

// Labels for CBA buttons etc.
_LIT(KBlankLabel, "");
_LIT(KNewCommandLabel, "New\ncommand");
_LIT(KEditCommandLabel, "Edit\ncommand");
_LIT(KRemoveCommandLabel, "Remove\ncommand");
_LIT(KRecordLabel, "Record");
_LIT(KRerecordLabel, "Re-record");
_LIT(KDoneLabel, "Done");
_LIT(KCancelLabel, "Cancel");
_LIT(KBrowseLabel, "Browse");
_LIT(KSelectLabel, "Select");
_LIT(KCloseLabel, "Close");

// Locations of the files containing recorded voice commands.
_LIT(KRawVoiceFile, "\\system\\data\\VoiceCommandTmp.raw");
_LIT(KCompactVoiceFile, "\\system\\data\\VoiceCommandTmp2.raw");

// Location of the voice daemon (hard-coded).
_LIT(KVoiceDaemonPath, "\\system\\apps\\VoiceDaemon\\VoiceDaemon.app");

// The main entry point for the dialog.
IMPORT_C void CreateControlL(const TDesC& aDllFullPathName);

// This is a standard dialog.
class CVoiceControlDialog : public CEikDialog
	{
public:
	// Basic C++ constructor and destructor.
	CVoiceControlDialog(TInt aResourceFileId);
	virtual ~CVoiceControlDialog();

	// Adds a new command to the list of commands.
	void AddCommandToListL(const CVoiceCommand* aCommand);
	// Returns the currently selected command in the list box.
	CVoiceCommand* GetSelectedCommand();
	// Updates the list of commands after one has changed.
	void UpdateCommandList();

	// Returns a pointer to the voice recorder object.
	CVoiceRecorder* VoiceRecorder();
	// Sets the drive the app is installed on. This is necessary to find
	// data files etc.
	void SetDriveLetter(const TDesC& aDriveLetter);
	// Returns the drive this app is installed on.
	const TDesC& DriveLetter();

	// from CCoeControl, used to implement context-sensitive help
	virtual void GetHelpContext(TCoeHelpContext& aContext) const;

private:
	// from CEikDialog
	// Called before the dialog is laid out.
	virtual void PreLayoutDynInitL();
	// Called when the user wishes to exit.
	virtual TBool OkToExitL(TInt aButtonId);
	// Called when a key is pressed/released.
	TKeyResponse OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType);
	// Sets the hot key to be used to execute a voice command.
	void SetHotKeyL(TUint aKeyCode, TUint aModifiers);
	// Called when the user switches between pages in the dialog.
	virtual void CVoiceControlDialog::PageChangedL(TInt aPageId);
	
	// from MCoeControlObserver
	// Called when an event of any kind happens to one of the controls.
	virtual void HandleControlEventL(CCoeControl* aControl, TCoeEvent aEventType);

	// Creates the "New command" dialog.
	void NewCommandL();
	// Creates the "Edit command" dialog.
	void EditCommandL();
	// Removes the currently selected command from the list.
	void RemoveCommandL();

	// The voice recorder object.
	CVoiceRecorder* iVoiceRecorder;

	// The ID of the resource file this dialog was created from.
	TInt iResourceFileId;
	// The letter of the drive that Voice Control is installed on.
	TBuf<10> iDriveLetter;

	// Pointers to the controls on the dialog, saved for convenience.
	CEikTextListBox* iListBox;
	CEikChoiceList* iEnableVoiceControl;
	CEikChoiceList* iEnablePlayTone;
	CEikChoiceList* iRepetitionsList;
	CEikEdwin* iHotKeyEditor;
	};



#endif
