// VoiceDaemon.h
//


#ifndef __VoiceDaemon_H
#define __VoiceDaemon_H

#include <coeccntx.h>

#include <eikappui.h>
#include <eikapp.h>
#include <eikdoc.h>
#include <eikmenup.h>

#include <eikon.hrh>
#include <eikenv.h>
#include <eikdialg.h>
#include <bautils.h>
#include <eikon.hrh>
#include <eiktxlbx.h>
#include <eiktxlbm.h>

#include <MdaAudioSampleEditor.h>
#include <MdaAudioTonePlayer.h>

#include <phoneclient.h>
#include <etel.h>

#include <VoiceDaemon.rsg>
#include "VoiceDaemon.hrh"

#include "VoiceRec.h"

const TUid KUidVoiceDaemon = { 0x01003003 };

_LIT(KRawVoiceFile, "\\system\\data\\VoiceDaemonTmp.raw");
_LIT(KCompactVoiceFile, "\\system\\data\\VoiceDaemonTmp2.raw");

class CVoiceDaemonAppUi;

class CVoiceDaemonDialer : public CActive
	{
public:
	enum TDialerState {
		EDialling = 0,
		EConnected,
		EHangingUp,
		EFinished
		};

	CVoiceDaemonDialer(CVoiceDaemonAppUi* aAppUi);
	virtual void StartL(const TDesC& aPhoneNumber);
	void HangUp();
	virtual void RunL();
	virtual void DoCancel();

private:
	CVoiceDaemonAppUi* iAppUi;
	TBuf<256> iPhoneNumber;
	RCall iPhoneCall;
	RCall::TStatus iPhoneCallStatus;
	RLine iPhoneLine;
	RPhone iPhone;
	RTelServer iTelServer;
	};

//
// class CVoiceDaemonAppView
//

class CVoiceDaemonDocument;

class CVoiceDaemonAppView : public CCoeControl, public MCoeControlBrushContext
	{
public:
	~CVoiceDaemonAppView();
	// FIXME: shouldn't call it iDoc
	void ConstructL(const TRect& aRect, CVoiceRecorder* aVoiceRecorder,
		CVoiceDaemonDocument *aDoc);
	void SetDisplayText(const TDesC& aText);

private: // from CCoeControl
	void Draw(const TRect& /*aRect*/) const;
	CVoiceRecorder* iVoiceRecorder;
	TInt iCaptureKeyHandle;
	HBufC* iDisplayText;
	CVoiceDaemonDocument *iDoc;
	};


//
// CVoiceDaemonAppUi
//

class CVoiceDaemonAppUi : public CEikAppUi
	{
public:
	void ConstructL();
	TKeyResponse HandleKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType);
	void SetDisplayText(const TDesC& aText);
	void CancelCaptureKey();
	void CaptureKey();
	~CVoiceDaemonAppUi();

private: // from CEikAppUi
	void HandleCommandL(TInt aCommand);

	TInt iCaptureKeyHandle;
	CVoiceDaemonAppView* iAppView;
	CVoiceRecorder* iVoiceRecorder;
	};


//
// CVoiceDaemonDocument
//

class CVoiceDaemonDocument : public CEikDocument, public MMdaObjectStateChangeObserver,
	public MMdaAudioToneObserver, public MExtPhoneObserver
	{
public:
	// construct/destruct
	CVoiceDaemonDocument(CEikApplication& aApp);
	~CVoiceDaemonDocument();
	static CVoiceDaemonDocument* NewL(CEikApplication& aApp);
	void ConstructL();

	CVoiceRecorder* VoiceRecorder();
	CMdaAudioToneUtility* MdaAudioToneUtility();
	TBool Calling();

	void ExecuteCommandL(CVoiceCommand* aCommand);
	void HangUpL();

	void BringToFront();
	void SendToBack();
	void PrepareForNextCommand();

	TBool iVoiceRecorderCreated;

private:
	// from CEikDocument
	CEikAppUi* CreateAppUiL();

	// from MMdaObjectStateChangeObserver
	void MoscoStateChangeEvent(CBase* aObject, TInt aPreviousState, TInt aCurrentState, TInt aErrorCode);

	CMdaAudioToneUtility* iMdaAudioToneUtility;
	virtual void MatoPrepareComplete(TInt aError);
	virtual void MatoPlayComplete(TInt aError);

	virtual void HandlePhoneStatusL(const TInt aStatus);
    virtual void HandleDialL(const TInt aStatus);

	void DialPhoneNumberL(const TDesC& aPhoneNumber);
	void DialContactL(const TDesC& aContactName, TBool aMobile);

	static TInt SplitContactWordsL(TAny* aPtr);

	TInt iState;

	CVoiceRecorder* iVoiceRecorder;
	CVoiceDaemonDialer* iDialer;

	TBool iReadyToRecord;

	RCall iPhoneCall;
	RLine iPhoneLine;
	RPhone iPhone;
	RTelServer iTelServer;
	TBool iCalling;
	};


//
// CVoiceDaemonApplication
//

class CVoiceDaemonApplication : public CEikApplication
	{
  private: // from CApaApplication
	CApaDocument* CreateDocumentL();
	TUid AppDllUid() const;
	};


#endif
