// This file defines the document class for the Voice Daemon app. This
// class contains the bulk of the functionality in the application, and
// is responsible for beeping, recording commands, and executing them.

#ifndef VOICE_DAEMON_DOCUMENT_H
#define VOICE_DAEMON_DOCUMENT_H

#include <eikappui.h>
#include <eikapp.h>
#include <eikdoc.h>
#include <MdaAudioTonePlayer.h>
#include "VoiceRec.h"

#include <phoneclient.h>

// The locations of the files the application uses to store recorded
// voice commands.
_LIT(KRawVoiceFile, "\\system\\data\\VoiceDaemonTmp.raw");
_LIT(KCompactVoiceFile, "\\system\\data\\VoiceDaemonTmp2.raw");

// Defintions for command buttons etc.
_LIT(KBlankLabel, "");
_LIT(KDoneLabel, "Done");
_LIT(KCancelLabel, "Cancel");
_LIT(KHangUpLabel, "Hang up");

class CVoiceDaemonDialer;

class CVoiceDaemonDocument : public CEikDocument, public MMdaObjectStateChangeObserver,
	public MMdaAudioToneObserver
	{
public:
	// construct/destruct
	CVoiceDaemonDocument(CEikApplication& aApp);
	~CVoiceDaemonDocument();
	static CVoiceDaemonDocument* NewL(CEikApplication& aApp);
	void ConstructL();

	CVoiceRecorder* VoiceRecorder();
	CMdaAudioToneUtility* MdaAudioToneUtility();
	void SetCalling(TBool aCalling);
	TBool Calling();

	void ExecuteCommandL(CVoiceCommand* aCommand);
	void HangUpL();
	void SetDisplayText(const TDesC& aText);

	void BringToFront();
	void SendToBack();
	void PrepareForNextCommandL();

	TBool iVoiceRecorderCreated;

private:
	// from CEikDocument
	CEikAppUi* CreateAppUiL();

	// from MMdaObjectStateChangeObserver
	void MoscoStateChangeEvent(CBase* aObject, TInt aPreviousState, TInt aCurrentState, TInt aErrorCode);

	CMdaAudioToneUtility* iMdaAudioToneUtility;
	virtual void MatoPrepareComplete(TInt aError);
	virtual void MatoPlayComplete(TInt aError);

	void DialPhoneNumberL(const TDesC& aPhoneNumber);
	TInt DialContactL(const TDesC& aContactName, TBool aMobile);

	static TInt SplitContactWordsL(TAny* aPtr);

	TInt iState;
	TBool iCalling;

	CVoiceRecorder* iVoiceRecorder;
	CVoiceDaemonDialer* iDialer;
	CEikonEnv* iEikEnv;
//	CExtPhone* iExtPhone;
	};

#endif