#include <coesndpy.h>
#include <bassnd.h>
#include <lafcknsd.h>
#include <apgtask.h>
#include <apgcli.h>
#include <eikbtgpc.h>
#include <cntdb.h>
#include <cntfield.h>
#include <cntitem.h>
#include <cntfldst.h>

#include "VoiceDaemon.h"

CVoiceDaemonDialer::CVoiceDaemonDialer(CVoiceDaemonAppUi* aAppUi)
	: CActive(EPriorityNormal), iAppUi(aAppUi)
	{
	CActiveScheduler::Add(this);
	}

void CVoiceDaemonDialer::StartL(const TDesC& aPhoneNumber)
	{
	// The name of the TSY to use to make the call.
	_LIT(KTsyName, "Phonetsy.tsy");
	RTelServer::TPhoneInfo info;
	RPhone::TLineInfo lineInfo;
	TBuf<256> formatBuf;

	iPhoneNumber = aPhoneNumber;

	// Give the user some feedback.
	formatBuf.Copy(_L("Dialling "));
	formatBuf.Append(iPhoneNumber);
	formatBuf.Append(_L("..."));
	iAppUi->SetDisplayText(formatBuf);

	// Connect to the telephony server and load the TSY.
	User::LeaveIfError(iTelServer.Connect());
	User::LeaveIfError(iTelServer.LoadPhoneModule(KTsyName));

	// Get the details for the first (and only) phone.
	User::LeaveIfError(iTelServer.GetPhoneInfo(0, info));

	// Open the phone.
	User::LeaveIfError(iPhone.Open(iTelServer, info.iName));

	// Get the information for the voice line, line 0.
	User::LeaveIfError(iPhone.GetLineInfo(0, lineInfo));
	
	// Open the line. iName will now be "VoiceLine1".
	User::LeaveIfError(iPhoneLine.Open(iPhone, lineInfo.iName));

	// Open a new phone call.
	User::LeaveIfError(iPhoneCall.OpenNewCall(iPhoneLine));

	// Actually dial the specified number.
	iPhoneCall.Dial(iStatus, aPhoneNumber);

	SetActive();
	}

void CVoiceDaemonDialer::RunL()
	{
	TBuf<256> formatBuf;
	CEikButtonGroupContainer* cba = CEikonEnv::Static()->AppUiFactory()->ToolBar();

	RCall::TStatus callStatus;
	iPhoneCall.GetStatus(callStatus);

	if ((iStatus.Int() != KErrNone) && (callStatus != RCall::EStatusIdle))
		{
		HangUp();
		return;
		}
	
	switch (callStatus)
		{
	case RCall::EStatusConnected:
		formatBuf.Copy(_L("Connected to "));
		formatBuf.Append(iPhoneNumber);
		iAppUi->SetDisplayText(formatBuf);
	//	iPhoneCall.NotifyStatusChange(iStatus, iPhoneCallStatus);
		break;

	case RCall::EStatusRinging:
		formatBuf.Copy(iPhoneNumber);
		formatBuf.Append(_L(" is ringing"));
		iAppUi->SetDisplayText(formatBuf);
		break;

	case RCall::EStatusAnswering:
		formatBuf.Copy(iPhoneNumber);
		formatBuf.Append(_L(" is answering"));
		iAppUi->SetDisplayText(formatBuf);
		break;

	case RCall::EStatusIdle:
	//	iPhoneCall.NotifyStatusChangeCancel();
		iPhoneCall.Close();
		iPhoneLine.Close();
		iPhone.Close();
		iTelServer.Close();
		iAppUi->SetDisplayText(_L("Please wait..."));
		cba->SetCommandL(3, ECbaButtonBlank, _L(""));
		break;

	default:
		break;
		}
	}

void CVoiceDaemonDialer::HangUp()
	{
//	iPhoneCall.NotifyStatusChangeCancel();

	RCall::TStatus callStatus;
	iPhoneCall.GetStatus(callStatus);

	switch (callStatus)
		{
	case RCall::EStatusDialling:
		iPhoneCall.DialCancel();
		break;

	case RCall::EStatusConnected:
	case RCall::EStatusRinging:
	case RCall::EStatusAnswering:
	case RCall::EStatusConnecting:
		iPhoneCall.HangUp(iStatus);
		SetActive();
		break;

	default:
		break;
		}
	}

void CVoiceDaemonDialer::DoCancel()
	{
	HangUp();
	}

//
// EXPORTed functions
//

EXPORT_C CApaApplication* NewApplication()
	{
	return new CVoiceDaemonApplication;
	}

GLDEF_C TInt E32Dll(TDllReason)
	{
	return KErrNone;
	}


////////////////////////////////////////////////////////////////
//
// Application class, CVoiceDaemonApplication
//
////////////////////////////////////////////////////////////////

TUid CVoiceDaemonApplication::AppDllUid() const
	{
	return KUidVoiceDaemon;
	}

CApaDocument* CVoiceDaemonApplication::CreateDocumentL()
	{
	// Construct the document using its NewL() function, rather 
	// than using new(ELeave), because it requires two-phase
	// construction.
	return CVoiceDaemonDocument::NewL(*this);
	}


////////////////////////////////////////////////////////////////
//
// Document class, CVoiceDaemonDocument
//
////////////////////////////////////////////////////////////////

// Frequency and duration of the tone to be played
const TInt KSoundFrequency = 2500;
const TInt KSoundDuration = 500000;

// C++ constructor
CVoiceDaemonDocument::CVoiceDaemonDocument(CEikApplication& aApp)
		: CEikDocument(aApp)
	{
	}

CVoiceDaemonDocument* CVoiceDaemonDocument::NewL(CEikApplication& aApp)
	{
	CVoiceDaemonDocument* self = new (ELeave) CVoiceDaemonDocument(aApp);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop();
	return self;
	}

void CVoiceDaemonDocument::ConstructL()
	{
	SendToBack();

//	CEikonEnv::Static()->SetSystem(ETrue);

	TParse parse;
	parse.SetNoWild(Application()->AppFullName(), NULL, NULL);
	// We need to set the default path to be used for future file access, because
	// it is always initialised to c: and the voice control files may be stored
	// on the d: drive.
	TFileName fileName;
	fileName.Copy(parse.Drive());
	fileName.Append(_L("\\"));
	CEikonEnv::Static()->FsSession().SetSessionPath(fileName);

	iVoiceRecorder = new (ELeave) CVoiceRecorder(CEikonEnv::Static()->FsSession());
	iVoiceRecorder->InternalizeL();

	iMdaAudioToneUtility = CMdaAudioToneUtility::NewL(*this);
	PrepareForNextCommand();
	}

// All resources allocated in ConstructL() must be released in 
// the destructor.
CVoiceDaemonDocument::~CVoiceDaemonDocument()
	{
	delete iDialer;
	delete iVoiceRecorder;
	delete iMdaAudioToneUtility;
	}

CEikAppUi* CVoiceDaemonDocument::CreateAppUiL()
	{
    CVoiceDaemonAppUi* tempAppUi = new (ELeave) CVoiceDaemonAppUi;
	iDialer = new (ELeave) CVoiceDaemonDialer(tempAppUi);
	return tempAppUi;
	}

CVoiceRecorder* CVoiceDaemonDocument::VoiceRecorder()
	{
	return iVoiceRecorder;
	}

CMdaAudioToneUtility* CVoiceDaemonDocument::MdaAudioToneUtility()
	{
	return iMdaAudioToneUtility;
	}

void CVoiceDaemonDocument::MoscoStateChangeEvent(CBase* /*aObject*/, TInt /*aPreviousState*/, TInt aCurrentState, TInt /*aErrorCode*/)
	{
	iState = aCurrentState;
	
	switch (iState)
		{
	case KReadyToRecord:
		// Now we're ready to record, capture the hotkey
		((CVoiceDaemonAppUi*) iAppUi)->CaptureKey();
		break;

	default:
		break;
		}
	}

void CVoiceDaemonDocument::MatoPrepareComplete(TInt /*aError*/)
	{
	iMdaAudioToneUtility->SetVolume(iMdaAudioToneUtility->MaxVolume());

	iVoiceRecorder->CreateVoiceFileL(KRawVoiceFile(), *this);
	}
	
void CVoiceDaemonDocument::MatoPlayComplete(TInt /*aError*/)
	{
	iVoiceRecorder->RecordL();
	((CVoiceDaemonAppUi*) iAppUi)->SetDisplayText(_L("Recording..."));
	}

void CVoiceDaemonDocument::DialPhoneNumberL(const TDesC& aPhoneNumber)
	{
	iCalling = ETrue;

	// Start the asynchronous dialer object with the specified phone number
	iDialer->StartL(aPhoneNumber);

	// Update the buttons now that we're on the phone
	CEikButtonGroupContainer* cba = CEikonEnv::Static()->AppUiFactory()->ToolBar();
	cba->SetCommandL(0, ECbaButtonBlank, _L(""));
	cba->SetCommandL(3, ECbaButtonHangUp, _L("Hang up"));
	cba->SetDefaultCommand(ECbaButtonHangUp);
	cba->DrawNow();
	}

// This function splits a string (such as a name) into words. It assumes
// that any character which is non-alphanumeric is space and can hence be
// discarded. Its purpose is to enable the finding of contacts which are
// specified as a full name.
TInt CVoiceDaemonDocument::SplitContactWordsL(TAny* aPtr)
	{
	SFindInTextDefWordParser* parser = (SFindInTextDefWordParser*) aPtr;
	TInt wordStart = 0;
	TBool inWord = ETrue;
	TPtrC searchPtr(parser->iSearchString->Ptr(), parser->iSearchString->Length());

	for (TInt i = 0; i <= searchPtr.Length(); i++)
		{
		if (i == searchPtr.Length())
			{
			if (inWord)
				parser->iWordArray->AppendL(searchPtr.Mid(wordStart, i - wordStart));
			break;
			}
		if (TChar(searchPtr[i]).IsAlphaDigit())
			{
			if (!inWord)
				wordStart = i;
			inWord = ETrue;
			}
		else
			{
			if (inWord)
				parser->iWordArray->AppendL(searchPtr.Mid(wordStart, i - wordStart));
			inWord = EFalse;
			}
		}
	return KErrNone;
	}

void CVoiceDaemonDocument::DialContactL(const TDesC& aContactName, TBool aMobile)
	{
	// Create a descriptor array with an arbitrary granularity. This will
	// be used to store the words that make up the contact name we were
	// passed.
	CDesCArrayFlat* wordArray = new (ELeave) CDesCArrayFlat(2);
	CleanupStack::PushL(wordArray);

	// Parse the contact name we were passed into separate words.
	SFindInTextDefWordParser parser(&aContactName, wordArray);
	SplitContactWordsL(&parser);

	// Open the contacts database.
	CContactDatabase* db = CContactDatabase::OpenL();
	CleanupStack::PushL(db);

	//	CContactIdArray* contacts = db->FindInTextDefLC(*wordArray,
//		TCallBack(SplitContactWordsL)); 

	//CContactItemFieldDef* fields = new (ELeave) CContactItemFieldDef();
	//CleanupStack::PushL(fields);
	//TUid uid;
	//uid.iUid = KIntContactHintIsGivenName;
	//fields->AppendL(KUidContactFieldMatchAll);
	//CContactIdArray* contacts = NULL;
	//CContactIdArray* contacts = db->FindLC(_L("aTest"), fields);
	//CleanupStack::PopAndDestroy();

//	CContactItemFieldDef* contactItemFieldDef = new (ELeave) CContactItemFieldDef;

//	CleanupStack::PushL(contactItemFieldDef);

//	contactItemFieldDef->AppendL(KUidContactFieldMatchAll);

	/*const CContactIdArray* contacts = db->SortedItemsL();
	for (TInt i = 0; i < count; i++)
		{
		CContactItem* contact = db->ReadMinimalContactL((*contacts)[i]);
		CleanupStack::PushL(contact);

		CContactItemFieldSet& fieldSet = contact->CardFields();
		TInt fieldIndex = fieldSet.Find(KUidContactFieldFamilyName);
		if (fieldIndex < 0)
			{
			CleanupStack::PopAndDestroy(); // contact
			continue;
			}

		CContactItemField& field = fieldSet[fieldIndex];
		CContactTextField* text = field.TextStorage();
		if (!text->Text().CompareF(aContactName))
			{
			TBuf<256> buf;
			buf.Format(_L("Found contact %S"), &aContactName);
			CEikonEnv::Static()->InfoMsg(buf);
			TInt numFieldIndex = fieldSet.Find(KUidContactFieldPhoneNumber);
			if (numFieldIndex < 0)
				{
				CleanupStack::PopAndDestroy(); // contact
				continue;
				}

			CContactItemField& numField = fieldSet[numFieldIndex];
			CContactTextField* numText = numField.TextStorage();

			//buf.Format(_L("Found phone number %S"), &(numText->Text()));
			//CEikonEnv::Static()->InfoMsg(buf);

			DialPhoneNumberL(numText->Text());

			CleanupStack::PopAndDestroy(); // contact
			return;
			}
		CleanupStack::PopAndDestroy(); // contact
		}
		*/
	CEikonEnv::Static()->InfoMsg((*wordArray)[0]);
	TContactTextDefItem familyName(KUidContactFieldFamilyName);
	TContactTextDefItem givenName(KUidContactFieldGivenName);

	CContactTextDef* contactTextDef = CContactTextDef::NewL();
	CleanupStack::PushL(contactTextDef);

	contactTextDef->AppendL(givenName);
	contactTextDef->AppendL(familyName);

//	CContactIdArray* contacts = db->FindLC(aContactName, contactItemFieldDef);
	CContactIdArray* contacts = db->FindInTextDefLC(*wordArray, contactTextDef,
		TCallBack(SplitContactWordsL));

	if (contacts->Count() == 0)
		User::Leave(KErrNotFound);
	
	// Get the details of the first contact we found.
	CContactItem* contact = db->ReadContactLC((*contacts)[0]);

	CContactItemFieldSet& fieldSet = contact->CardFields();

	// Find the index of the phone number field. If the "mobile" flag is
	// specified, we specifically try to find the mobile number. Otherwise,
	// we just use the standard phone number field. Note that this will return
	// the first mobile number if no land line number exists.
	TInt numFieldIndex;
	if (!aMobile)
		numFieldIndex = fieldSet.Find(KUidContactFieldPhoneNumber);
	else
		numFieldIndex = fieldSet.Find(KUidContactFieldVCardMapCELL);

	if (numFieldIndex < 0)
		User::Leave(KErrNotFound);

	CContactItemField& numField = fieldSet[numFieldIndex];
	CContactTextField* numText = numField.TextStorage();

	//buf.Format(_L("Found phone number %S"), &(numText->Text()));
	//CEikonEnv::Static()->InfoMsg(buf);

	DialPhoneNumberL(numText->Text());

	CleanupStack::PopAndDestroy(5); // db, wordArray, contactTextDef, contacts, contact
	}

void CVoiceDaemonDocument::HangUpL()
	{
	iCalling = EFalse;
	iDialer->HangUp();
	SendToBack();
	}

void CVoiceDaemonDocument::ExecuteCommandL(CVoiceCommand* aCommand)
	{
	TBuf<256> formatBuf;

	switch (aCommand->iCommand)
		{
	case EVoiceCommandDialContact:
		DialContactL(*(aCommand->iArgument), EFalse);
		break;

	case EVoiceCommandDialContactMobile:
		DialContactL(*(aCommand->iArgument), ETrue);
		break;

	case EVoiceCommandDialNumber:
		DialPhoneNumberL(*(aCommand->iArgument));
		break;

	case EVoiceCommandOpenDocument:
		{
		formatBuf.Copy(_L("Trying to open "));
		formatBuf.Append(*(aCommand->iArgument));
		((CVoiceDaemonAppUi*) iAppUi)->SetDisplayText(formatBuf);

		RApaLsSession lsSession;
		TThreadId appId;
		User::LeaveIfError(lsSession.Connect());
		lsSession.StartDocument(*(aCommand->iArgument), appId);
		lsSession.Close();

		// FIXME: this puts the most recently opened app behind the desk
		//SendToBack();
		}
		break;

	case EVoiceCommandOpenApplication:
		{
		RApaLsSession lsSession;
		User::LeaveIfError(lsSession.Connect());
		CApaCommandLine* cmdLine = CApaCommandLine::NewL(*(aCommand->iArgument));
		lsSession.StartApp(*cmdLine);
		lsSession.Close();
		delete cmdLine;

		//SendToBack();
		}
		break;

	default:
		((CVoiceDaemonAppUi*) iAppUi)->SetDisplayText(_L("Unrecognised command"));
		break;
		}
	}

void CVoiceDaemonDocument::HandlePhoneStatusL(const TInt aStatus)
	{
	TBuf<100> buf;
	buf.Format(_L("Phone status: %d"), aStatus);
	CEikonEnv::Static()->InfoMsg(buf);
	}

void CVoiceDaemonDocument::HandleDialL(const TInt aStatus)
	{
	TBuf<100> buf;
	buf.Format(_L("Dial status: %d"), aStatus);
	CEikonEnv::Static()->InfoMsg(buf);
	}

void CVoiceDaemonDocument::BringToFront()
	{
	if (iAppUi != NULL)
		((CVoiceDaemonAppUi*) iAppUi)->SetDisplayText(_L("Please wait..."));

	TApaTask task(CEikonEnv::Static()->WsSession());
	task.SetWgId(CEikonEnv::Static()->RootWin().Identifier());
	task.BringToForeground();
	}

void CVoiceDaemonDocument::SendToBack()
	{
	TApaTask task(CEikonEnv::Static()->WsSession());
	task.SetWgId(CEikonEnv::Static()->RootWin().Identifier());
	task.SendToBackground();
	
	if (iAppUi != NULL)
		((CVoiceDaemonAppUi*) iAppUi)->SetDisplayText(_L(""));
	}

void CVoiceDaemonDocument::PrepareForNextCommand()
	{
	if (iVoiceRecorder->EnablePlayTone())
		{
		iMdaAudioToneUtility->PrepareToPlayTone(KSoundFrequency,
			TTimeIntervalMicroSeconds(KSoundDuration));
		}
	else
		{
		iVoiceRecorder->CreateVoiceFileL(KRawVoiceFile(), *this);
		}
	}

TBool CVoiceDaemonDocument::Calling()
	{
	return iCalling;
	}

////////////////////////////////////////////////////////////////
//
// App UI class, CVoiceDaemonAppUi
//
////////////////////////////////////////////////////////////////

void CVoiceDaemonAppUi::ConstructL()
    {
    BaseConstructL();
	iVoiceRecorder = ((CVoiceDaemonDocument*) iDocument)->VoiceRecorder();

    iAppView = new(ELeave) CVoiceDaemonAppView;
    iAppView->ConstructL(ClientRect(), iVoiceRecorder, ((CVoiceDaemonDocument*) iDocument));
	
	SetDisplayText(_L("Please wait"));

	AddToStackL(iAppView);
    }

CVoiceDaemonAppUi::~CVoiceDaemonAppUi()
	{
    delete iAppView;
	}

void CVoiceDaemonAppUi::HandleCommandL(TInt aCommand)
	{
	CVoiceDaemonDocument* doc = ((CVoiceDaemonDocument*) iDocument);
	CVoiceCommand* cmd;
	CVoiceCommand* closestCmd;
	TInt rc;

	switch (aCommand)
		{
	case ECbaButtonHangUp:
		doc->HangUpL();
		break;

	case ECbaButtonStopRecording:
		doc->VoiceRecorder()->Stop();
		SetDisplayText(_L("Finding command..."));

		// Compress the file containing the recorded voice, so that it's
		// more easy to work with. This will return KErrNotFound if the
		// file just contains silence.
		rc = doc->VoiceRecorder()->CompactVoiceFileL(KCompactVoiceFile(),
			KRawVoiceFile());
		if (rc == KErrNotFound)
			{
			CEikonEnv::Static()->AlertWin(_L("No speech was recorded."),
				_L("Try speaking more loudly."));
			doc->SendToBack();
			}
		else
			{
			cmd = new (ELeave) CVoiceCommand;
			CleanupStack::PushL(cmd);

			doc->VoiceRecorder()->CreateSignatureL(KCompactVoiceFile(), cmd, 0);
			closestCmd = doc->VoiceRecorder()->FindClosestVoiceCommandL(cmd);

			if (closestCmd == NULL)
				{
				CEikonEnv::Static()->AlertWin(_L("No matching command found"));
				doc->SendToBack();
				}
			else
				{
				doc->ExecuteCommandL(closestCmd);
				}
			CleanupStack::PopAndDestroy(); // cmd
			}
		doc->PrepareForNextCommand();
		break;

	case ECbaButtonCancelRecording:
		doc->SendToBack();
		doc->VoiceRecorder()->Stop();
		doc->PrepareForNextCommand();
		break;

	case EEikCmdExit:
		Exit();
		break;

	default:
		break;
		}
	}

TKeyResponse CVoiceDaemonAppUi::HandleKeyEventL(const TKeyEvent& aKeyEvent,TEventCode aType)
	{
	CVoiceDaemonDocument* doc = ((CVoiceDaemonDocument*) iDocument);

	// We're only interested on a key up event when we're
	// not recording or on the phone.
	if ((aType != EEventKeyUp) || !iCaptureKeyHandle || doc->Calling())
		return EKeyWasNotConsumed;

	TInt hotKeyScanCode;
	TUint hotKeyModifiers;
	iVoiceRecorder->GetHotKey(hotKeyScanCode, hotKeyModifiers);

	if ((aKeyEvent.iScanCode == hotKeyScanCode) && (aKeyEvent.iModifiers & hotKeyModifiers))
		{
		CancelCaptureKey();

		doc->BringToFront();

		//TBuf<100> buf;
		//buf.Format(_L("event: %X (%c) mods: %X scan: %d type: %X"), aKeyEvent.iCode, aKeyEvent.iCode,
		//	aKeyEvent.iModifiers, aKeyEvent.iScanCode, aType);
		//CEikonEnv::Static()->InfoMsg(buf);

		//doc->MdaAudioToneUtility()->PrepareToPlayFileSequence(_L("z:\\system\\tones\\beep.rng"));
		if (doc->VoiceRecorder()->EnablePlayTone())
			{
			doc->MdaAudioToneUtility()->Play();
			}
		else
			{
			doc->VoiceRecorder()->RecordL();
			SetDisplayText(_L("Recording..."));
			}

		CEikButtonGroupContainer* cba = CEikonEnv::Static()->AppUiFactory()->ToolBar();
		cba->SetCommandL(0, ECbaButtonStopRecording, _L("Done"));
		cba->SetCommandL(1, ECbaButtonBlank, _L(""));
		cba->SetCommandL(2, ECbaButtonBlank, _L(""));
		cba->SetCommandL(3, ECbaButtonCancelRecording, _L("Cancel"));
		cba->SetDefaultCommand(ECbaButtonStopRecording);
		cba->DrawNow();

		return EKeyWasConsumed;
		}
	return EKeyWasNotConsumed;
	}

void CVoiceDaemonAppUi::CaptureKey()
	{
	TInt hotKeyScanCode;
	TUint hotKeyModifiers;
	iVoiceRecorder->GetHotKey(hotKeyScanCode, hotKeyModifiers);

	iCaptureKeyHandle = CEikonEnv::Static()->RootWin().CaptureKeyUpAndDowns(hotKeyScanCode,
		hotKeyModifiers, hotKeyModifiers);
	}

void CVoiceDaemonAppUi::CancelCaptureKey()
	{
	CEikonEnv::Static()->RootWin().CancelCaptureKeyUpAndDowns(iCaptureKeyHandle);
	iCaptureKeyHandle = 0;
	}

void CVoiceDaemonAppUi::SetDisplayText(const TDesC& aDisplayText)
	{
	iAppView->SetDisplayText(aDisplayText);
	iAppView->DrawNow();
	}

////////////////////////////////////////////////////////////////
//
// Application view class, CVoiceDaemonAppView
//
////////////////////////////////////////////////////////////////

void CVoiceDaemonAppView::ConstructL(const TRect& aRect, CVoiceRecorder* aVoiceRecorder,
									 CVoiceDaemonDocument* aDoc)
    {
	iVoiceRecorder = aVoiceRecorder;
	iDoc = aDoc;
    CreateWindowL();
    SetRect(aRect);
    ActivateL();

	iDisplayText = HBufC::NewL(0);
    }

void CVoiceDaemonAppView::Draw(const TRect& /*aRect*/) const
	{
	CWindowGc& gc = SystemGc();
	// Clear the application view
	gc.Clear();

	const CFont* fontUsed;

    // Use the title font supplied by the UI
	fontUsed = CEikonEnv::Static()->TitleFont();
	gc.UseFont(fontUsed);

	// Draw the text in the middle of the view rectangle.
	TInt baselineOffset = (Rect().Height() - fontUsed->HeightInPixels()) / 2; 
	gc.DrawText(*iDisplayText, Rect(), baselineOffset, CGraphicsContext::ECenter, 0);

    // Finished using the font
	gc.DiscardFont();
	}

void CVoiceDaemonAppView::SetDisplayText(const TDesC& aText)
	{
	delete iDisplayText;
	iDisplayText = NULL;
	iDisplayText = aText.AllocL();
	}

CVoiceDaemonAppView::~CVoiceDaemonAppView()
	{
	CEikonEnv::Static()->RootWin().CancelCaptureKeyUpAndDowns(iCaptureKeyHandle);
	delete iDisplayText;
	}