#include <eikbtgpc.h>
#include <apgwgnam.h>

#include "VoiceDaemonAppUi.h"

////////////////////////////////////////////////////////////////
//
// App UI class, CVoiceDaemonAppUi
//
////////////////////////////////////////////////////////////////

void CVoiceDaemonAppUi::ConstructL()
    {
	// Perform necessary construction in CEikAppUi.
    BaseConstructL();

	// Get a pointer to the voice recorder object.
	iVoiceRecorder = ((CVoiceDaemonDocument*) iDocument)->VoiceRecorder();

	// Create the application view.
    iAppView = new(ELeave) CVoiceDaemonAppView;
    iAppView->ConstructL(ClientRect(), iVoiceRecorder, ((CVoiceDaemonDocument*) iDocument));
	
	// Set the initial text to be displayed in the view.
	_LIT(KPressHotkey, "Press the hotkey to record");
	SetDisplayText(KPressHotkey);
	
	// Enable the processing of commands in the app view.
	AddToStackL(iAppView);

	// Uncomment these lines to set the application's window group
	// name to be hidden, so the app is not displayed in the task list.
	// I didn't enable this because a) it then becomes difficult to kill
	// the app. and b) it's quite annoying not to know if it's running.
//	CApaWindowGroupName* wgName = CApaWindowGroupName::NewL(
//		iEikonEnv->WsSession(),
//		(TUint32)&iEikonEnv->RootWin());
//	wgName->SetHidden(ETrue);
//	wgName->SetWindowGroupName(iEikonEnv->RootWin());
    }

CVoiceDaemonAppUi::~CVoiceDaemonAppUi()
	{
	CancelCaptureKey();
    delete iAppView;
	}

// This function will be called by the framework when the user presses
// one of the CBA buttons.
void CVoiceDaemonAppUi::HandleCommandL(TInt aCommand)
	{
	CVoiceDaemonDocument* doc = ((CVoiceDaemonDocument*) iDocument);
	CEikButtonGroupContainer* cba = iEikonEnv->AppUiFactory()->ToolBar();
	CVoiceCommand* cmd;
	CVoiceCommand* closestCmd;
	TInt rc;

	_LIT(KFindingCommand, "Finding command...");

	switch (aCommand)
		{
	case ECbaButtonHangUp:
		doc->HangUpL();
		break;

	case ECbaButtonStopRecording:
		// Actually stop recording.
		doc->VoiceRecorder()->Stop();

		// Don't let the user press that button again.
		cba->SetCommandL(0, ECbaButtonBlank, KBlankLabel);
		cba->DrawNow();
		
		SetDisplayText(KFindingCommand);

		// Compress the file containing the recorded voice, so that it's
		// more easy to work with. This will return KErrNotFound if the
		// file just contains silence.
		rc = doc->VoiceRecorder()->CompactVoiceFileL(KCompactVoiceFile(),
			KRawVoiceFile());
		if (rc == KErrNotFound)
			{
			// Display an alert window, then just send the application
			// to the back again.
			_LIT(KNoSpeechRecorded, "No speech was recorded.");
			_LIT(KSpeakMoreLoudly, "Try speaking more loudly.");
			iEikonEnv->AlertWin(KNoSpeechRecorded, KSpeakMoreLoudly);
			doc->SendToBack();
			}
		else
			{
			// Create a new CVoiceCommand object to be used to store
			// information about command that's just been recorded.
			cmd = new (ELeave) CVoiceCommand;
			CleanupStack::PushL(cmd);

			// Create a signature for the voice command.
			doc->VoiceRecorder()->CreateSignatureL(KCompactVoiceFile(), cmd, 0);

			// Find the command which most closely approximates the one
			// that's just been recorded.
			closestCmd = doc->VoiceRecorder()->FindClosestVoiceCommandL(cmd);

			// Execute the command, if we found one. Otherwise, just inform
			// the user and go to the back again.
			if (closestCmd == NULL)
				{
				_LIT(KNoCommandFound, "No matching command found.");
				_LIT(KCheckSettings, "Check the settings in Control Panel.");
				iEikonEnv->AlertWin(KNoCommandFound, KCheckSettings);
				doc->SendToBack();
				}
			else
				{
				doc->ExecuteCommandL(closestCmd);
				}
			CleanupStack::PopAndDestroy(); // cmd
			}

		// Prepare to record another command.
		doc->PrepareForNextCommandL();
		break;

	case ECbaButtonCancelRecording:
		doc->SendToBack();
		// Disable all the buttons.
		cba->SetCommandL(0, ECbaButtonBlank, KBlankLabel);
		cba->SetCommandL(3, ECbaButtonBlank, KBlankLabel);
		doc->VoiceRecorder()->Stop();
		doc->PrepareForNextCommandL();
		break;

	case EEikCmdExit:
		Exit();
		break;

	default:
		break;
		}
	}

TKeyResponse CVoiceDaemonAppUi::HandleKeyEventL(const TKeyEvent& aKeyEvent,
												TEventCode aType)
	{
	CVoiceDaemonDocument* doc = ((CVoiceDaemonDocument*) iDocument);

	// We're only interested on a key up event when we're
	// not recording or on the phone.
	if ((aType != EEventKeyUp) || !iCaptureKeyHandle || doc->Calling())
		return EKeyWasNotConsumed;

	// Find out which hot key we're looking for.
	TInt hotKeyScanCode;
	TUint hotKeyModifiers;
	iVoiceRecorder->GetHotKey(hotKeyScanCode, hotKeyModifiers);

	// If we just caught the right hotkey, start recording.
	if ((aKeyEvent.iScanCode == hotKeyScanCode) && (aKeyEvent.iModifiers & hotKeyModifiers))
		{
		// Stop capturing the hotkey until the command has finished being
		// recorded.
		CancelCaptureKey();

		// Bring the application to the front.
		doc->BringToFront();

		// If we need to play a tone before recording, do so. Otherwise,
		// just start recording immediately.
		if (doc->VoiceRecorder()->EnablePlayTone())
			{
			doc->MdaAudioToneUtility()->Play();
			}
		else
			{
			doc->VoiceRecorder()->RecordL();
			_LIT(KRecording, "Recording...");
			SetDisplayText(KRecording);
			}

		// Update the CBA now that we're recording (or about to).
		CEikButtonGroupContainer* cba = iEikonEnv->AppUiFactory()->ToolBar();
		cba->SetCommandL(0, ECbaButtonStopRecording, KDoneLabel);
		cba->SetCommandL(1, ECbaButtonBlank, KBlankLabel);
		cba->SetCommandL(2, ECbaButtonBlank, KBlankLabel);
		cba->SetCommandL(3, ECbaButtonCancelRecording, KCancelLabel);
		cba->SetDefaultCommand(ECbaButtonStopRecording);
		cba->DrawNow();

		// We consumed this hotkey, so don't pass it on to any other
		// applications.
		return EKeyWasConsumed;
		}

	// We didn't consume the key event.
	return EKeyWasNotConsumed;
	}

void CVoiceDaemonAppUi::CaptureKey()
	{
	// Find out which hotkey we need to capture.
	TInt hotKeyScanCode;
	TUint hotKeyModifiers;
	iVoiceRecorder->GetHotKey(hotKeyScanCode, hotKeyModifiers);

	// Notify us when the hotkey is pressed or released. Note that we
	// don't use CaptureKey because we would get repeated events, and
	// also because we need to use scancodes, not keys. We store the
	// handle from the key capture so we can cancel it later.
	iCaptureKeyHandle = iEikonEnv->RootWin().CaptureKeyUpAndDowns(hotKeyScanCode,
		hotKeyModifiers, hotKeyModifiers);
	}

void CVoiceDaemonAppUi::CancelCaptureKey()
	{
	// Cancel the key capture using the handle we saved earlier.
	iEikonEnv->RootWin().CancelCaptureKeyUpAndDowns(iCaptureKeyHandle);
	iCaptureKeyHandle = 0;
	}

void CVoiceDaemonAppUi::SetDisplayText(const TDesC& aDisplayText)
	{
	// Set the view's text to display and redraw it.
	iAppView->SetDisplayText(aDisplayText);
	iAppView->DrawNow();
	}